@@ -6,7 +6,7 @@
 # This library implements OpenID Authentication 2.0,
 # in the role of a relying party
 
-import urlparse, urllib, httplib, time, cgi, htmllib, formatter
+import urlparse, urllib, httplib, time, cgi, HTMLParser
 import cStringIO, base64, hmac, hashlib, datetime, re, random
 import itertools, cPickle, sys
 
@@ -26,9 +26,17 @@ except ImportError:
 if sys.version_info < (3,):
     def b(s):
         return s
+    # Convert byte to integer
+    b2i = ord
+    def bytes_from_ints(L):
+        return ''.join([chr(i) for i in L])
 else:
     def b(s):
-        return s.encode('ascii')
+        return s.encode('latin-1')
+    def b2i(char):
+        # 3.x: bytes are already sequences of integers
+        return char
+    bytes_from_ints = bytes
 
 def normalize_uri(uri):
     """Normalize an uri according to OpenID section 7.2. Return a pair
@@ -138,24 +146,24 @@ def parse_response(s):
         res[k] = v
     return res
 
-class OpenIDParser(htmllib.HTMLParser):
+class OpenIDParser(HTMLParser.HTMLParser):
     def __init__(self):
-        htmllib.HTMLParser.__init__(self, formatter.NullFormatter())
+        HTMLParser.HTMLParser.__init__(self)
         self.links = {}
         self.xrds_location=None
 
-    def do_link(self, attrs):
-        attrs = dict(attrs)
-        try:
-            self.links[attrs['rel']] = attrs['href']
-        except KeyError:
-            pass
-
-    def do_meta(self, attrs):
-        attrs = dict(attrs)
-        # Yadis 6.2.5 option 1: meta tag
-        if attrs.get('http-equiv','').lower() == 'x-xrds-location':
-            self.xrds_location = attrs['content']
+    def handle_starttag(self, tag, attrs):
+        if tag == 'link':
+            attrs = dict(attrs)
+            try:
+                self.links[attrs['rel']] = attrs['href']
+            except KeyError:
+                pass
+        elif tag == 'meta':
+            attrs = dict(attrs)
+            # Yadis 6.2.5 option 1: meta tag
+            if attrs.get('http-equiv','').lower() == 'x-xrds-location':
+                self.xrds_location = attrs['content']
 
 def _extract_services(doc):
     for svc in doc.findall(".//{xri://$xrd*($v*2.0)}Service"):
@@ -241,7 +249,7 @@ def discover(url):
 
     if content_type in ('text/html', 'application/xhtml+xml'):
         parser = OpenIDParser()
-        parser.feed(data)
+        parser.feed(data.decode('latin-1'))
         parser.close()
         # Yadis 6.2.5 option 1: meta tag
         if parser.xrds_location:
@@ -273,22 +281,29 @@ def resolve_xri(xri, proxy='xri.net'):
     Return canonical ID, services, op endpoint, op local;
     return None if an error occurred'''
     xri = urllib.quote(xri, safe='=@*!+()')
-    conn = httplib.HTTPConnection(proxy)
-    try:
-        conn.connect()
-    except:
-        # DNS or TCP error
-        return None
-    conn.putrequest("GET", '/'+xri+'?_xrd_r=application/xrds+xml')
-    conn.endheaders()
-
-    res = conn.getresponse()
-    data = res.read()
-    conn.close()
-
-    doc = ElementTree.fromstring(data)
-    res = _extract_services(doc)
-    if res is None:
+    # Ask explicitly for the specific service types, to
+    # avoid having to identify the correct XRD element in
+    # case the identifier is hierarchical
+    for stype in ('http://specs.openid.net/auth/2.0/signon',
+                  'http://openid.net/signon/1.0'):
+        conn = httplib.HTTPConnection(proxy)
+        try:
+            conn.connect()
+        except:
+            # DNS or TCP error
+            return None
+        conn.putrequest("GET", '/'+xri+'?_xrd_r=application/xrd+xml'
+                        +'&_xrd_t='+stype)
+        conn.putheader('Connection', 'Keep-Alive')
+        conn.endheaders()
+        res = conn.getresponse()
+        data = res.read()
+        doc = ElementTree.fromstring(data)
+        res = _extract_services(doc)
+        conn.close()
+        if res is not None:
+            break
+    else:
         # No OpenID service found
         return None
     services, op_endpoint, op_local = res
@@ -324,13 +339,13 @@ def btwoc(l):
     res = cPickle.dumps(l, 2)
     # Pickle result: proto 2, long1 (integer < 256 bytes)
     # number of bytes, little-endian integer, stop
-    assert res[:3] == '\x80\x02\x8a' 
+    assert res[:3] == b('\x80\x02\x8a')
     # btwoc ought to produce the shortest representation in two's
     # complement. Fortunately, pickle already does that.
-    return res[3+ord(res[3]):3:-1]
+    return res[3+b2i(res[3]):3:-1]
 
-def unbtwoc(b):
-    return cPickle.loads('\x80\x02\x8a'+chr(len(b))+b[::-1]+'.')
+def unbtwoc(B):
+    return cPickle.loads(b('\x80\x02\x8a')+bytes_from_ints([len(B)])+B[::-1]+(b'.'))
 
 # Appendix B; DH default prime
 dh_prime = """
@@ -344,8 +359,8 @@ dh_prime = long("".join(dh_prime.split()), 16)
 def string_xor(s1, s2):
     res = []
     for c1, c2 in itertools.izip(s1, s2):
-        res.append(chr(ord(c1) ^ ord(c2)))
-    return ''.join(res)
+        res.append(b2i(c1) ^ b2i(c2))
+    return bytes_from_ints(res)
 
 def associate(services, url):
     '''Create an association (OpenID section 8) between RP and OP.
@@ -371,25 +386,25 @@ def associate(services, url):
         if data['openid.session_type'] == "no-encryption":
             data['openid.session_type'] = ''
         del data['openid.ns']
-    res = urllib.urlopen(url, urllib.urlencode(data))
+    res = urllib.urlopen(url, b(urllib.urlencode(data)))
     if res.getcode() != 200:
         raise ValueError, "OpenID provider refuses connection with status %d" % res.getcode()
     data = parse_response(res.read())
     if 'error' in data:
         raise ValueError, "associate failed: "+data['error']
     if url.startswith('http:'):
-        enc_mac_key = data.get('enc_mac_key')
+        enc_mac_key = b(data.get('enc_mac_key'))
         if not enc_mac_key:
             raise ValueError, "Provider protocol error: not using DH-SHA1"
-        enc_mac_key = base64.b64decode(data['enc_mac_key'])
-        dh_server_public = unbtwoc(base64.b64decode(data['dh_server_public']))
+        enc_mac_key = base64.b64decode(enc_mac_key)
+        dh_server_public = unbtwoc(base64.b64decode(b(data['dh_server_public'])))
         # shared secret: sha1(2^(server_priv*priv) mod prime) xor enc_mac_key
         shared_secret = btwoc(pow(dh_server_public, priv, dh_prime))
         shared_secret = hashlib.sha1(shared_secret).digest()
         if len(shared_secret) != len(enc_mac_key):
             raise ValueError, "incorrect DH key size"
         # Fake mac_key result
-        data['mac_key'] = base64.b64encode(string_xor(enc_mac_key, shared_secret))
+        data['mac_key'] = b(base64.b64encode(string_xor(enc_mac_key, shared_secret)))
     return data
 
 class _AX:
@@ -540,7 +555,7 @@ def authenticate(session, response):
         query.append(value)
     query = b('').join(query)
 
-    mac_key = base64.decodestring(b(session['mac_key']))
+    mac_key = base64.decodestring(session['mac_key'])
     transmitted_sig = base64.decodestring(b(response['openid.sig'][0]))
     computed_sig = hmac.new(mac_key, query, hashlib.sha1).digest()
 
