@@ -5,6 +5,7 @@
 
 from django.conf import settings
 from django.contrib.auth.models import User
+from django.contrib.auth.backends import ModelBackend
 from openid2rp.django.models import UserOpenID, OpenIDSession, OpenIDNonce
 from django.http import HttpResponse
 from django.db.models import Q
@@ -125,7 +126,8 @@ def unlinkOpenID(user, openid):
 
 def preAuthenticate(uri, answer_url, 
 					sreg = (('nickname', 'email'), ()),
-					ax = ((openid2rp.AX.email, openid2rp.AX.first, openid2rp.AX.last), ())):
+					ax = ((openid2rp.AX.email, openid2rp.AX.first, openid2rp.AX.last), ()),
+					reuse_session = True):
 
 	cleanup()
 	try:
@@ -136,7 +138,12 @@ def preAuthenticate(uri, answer_url,
 	if res != None:
 		services, url, op_local = res
 		# re-use session in order to avoid provider roundtrip here
-		session = getSessionByClaim(claimedId)
+		# some providers (Wordpress) do not like that, and send 'invalidate_handle' then,
+		# which would need another user roundtrip - therefore, the app can switch it off
+		if reuse_session:
+			session = getSessionByClaim(claimedId)
+		else:
+			session = None
 		if not session:
 			session = openid2rp.associate(services, url)
 			storeSession(session, claimedId)
@@ -148,7 +155,11 @@ def preAuthenticate(uri, answer_url,
 	else:
 		raise IncorrectClaimError()
 
-class Backend:	
+class Backend(ModelBackend):	
+	supports_object_permissions = False
+	supports_anonymous_user = True
+	supports_inactive_user = True
+	
 	def get_user(self, user_id):
 		try:
 			return User.objects.get(pk=user_id)
@@ -172,14 +183,15 @@ class Backend:
 			signed=openid2rp.authenticate(session, query)
 		except Exception, e:
 			raise AuthenticationError(str(e))
-		# check for replay attack
-		nonce = request.GET['openid.response_nonce']
-		timestamp = openid2rp.parse_nonce(nonce)		
-		if timestamp < datetime.datetime.utcnow() - maxTimeShift - maxLoginDelay: 
-			raise TookTooLongError()
-		elif knownNonce(nonce):
-			raise ReplayAttackError()
-		storeNonce(nonce)
+		# check for replay attack, only available with OpenID 2 providers
+		if 'openid.response_nonce' in request.GET:
+			nonce = request.GET['openid.response_nonce']
+			timestamp = openid2rp.parse_nonce(nonce)		
+			if timestamp < datetime.datetime.utcnow() - maxTimeShift - maxLoginDelay: 
+				raise TookTooLongError()
+			elif knownNonce(nonce):
+				raise ReplayAttackError()
+			storeNonce(nonce)
 
 		# provider-based auth returns claim id, OpenID not (if I got that right) - in this case we take the one stored in the session found by assocHandle
 		if 'openid.claimed_id' in request.GET:
@@ -189,7 +201,7 @@ class Backend:
 		else:
 			if 'identity' not in signed:
 				raise IncompleteAnswerError()
-			claimedId = session.claimedId
+			claimedId = session['claimedId']
 
 		# look up OpenID claim string in local database
 		idrecord=UserOpenID.objects.filter(Q(uri=claimedId))
